from . import channel_order, color, duration, gamma, ledtype

FIELD_TYPES = {
    'c_order': channel_order,
    'color': color,
    'duration': duration,
    'gamma': gamma,
    'ledtype': ledtype,
    'time': duration,
    'ledtype': ledtype,
}
